'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports['default'] = createCRUD;

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj['default'] = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _immutable = require('immutable');

var _actions = require('../actions');

var _utils = require('../utils');

var utils = _interopRequireWildcard(_utils);

var _constants = require('../constants');

var INITIAL_STATE = (0, _immutable.Map)({
  folders: (0, _immutable.Map)(),
  documents: (0, _immutable.Map)()
});

exports.INITIAL_STATE = INITIAL_STATE;
/**
 * Creates CRUD action types, actions and reducer for PouchDB database.
 *
 * @param {PouchDB} db - PouchDB instance
 * @param {string} [mountPoint] - prefix to use for action types
 * @param {string} [prefix=mountPoint] - prefix to use for action types
 * @param {Object} [opts={}] options
 *
 * @returns {Object} contains `actionTypes`, `actions`, `reducer`
 */

function createCRUD(db, mountPoint) {
  var prefix = arguments.length <= 2 || arguments[2] === undefined ? null : arguments[2];
  var opts = arguments.length <= 3 || arguments[3] === undefined ? {} : arguments[3];

  (0, _invariant2['default'])(typeof db !== 'undefined', 'db is required');
  prefix = prefix || mountPoint;
  var urlPrefix = '/' + prefix;

  var defaultStartkey = mountPoint + '-';
  var defaultEndkey = mountPoint + '-￿';
  var _opts$startkey = opts.startkey;
  var startkey = _opts$startkey === undefined ? defaultStartkey : _opts$startkey;
  var _opts$endkey = opts.endkey;
  var endkey = _opts$endkey === undefined ? defaultEndkey : _opts$endkey;

  var actionTypes = {};
  for (var action in _constants.ACTIONS) {
    actionTypes[action] = {};
    for (var type in _constants.TYPES) {
      actionTypes[action][type] = utils.createActionType(prefix, action, type);
    };
  };

  var allDocs = function allDocs(folder, params, opts) {
    if (folder === undefined) folder = '';

    var mergedParams = _extends({
      attachments: true,
      include_docs: true
    }, params);
    return (0, _actions.createPromiseAction)(function () {
      return db.allDocs(mergedParams);
    }, actionTypes.allDocs, _extends({}, opts, { folder: folder }));
  };

  var query = function query(fun, folder, params, opts) {
    if (folder === undefined) folder = '';

    var mergedParams = _extends({
      attachments: true,
      include_docs: true
    }, params);
    return (0, _actions.createPromiseAction)(function () {
      return db.query(fun, mergedParams);
    }, actionTypes.query, _extends({}, opts, { folder: folder }));
  };

  var get = function get(docId, params, opts) {
    if (params === undefined) params = {};

    return (0, _actions.createPromiseAction)(function () {
      return db.get(docId, params);
    }, actionTypes.get, _extends({}, opts, { docId: docId }));
  };

  var put = function put(doc, params, opts) {
    return (0, _actions.createPromiseAction)(function () {
      return db.put(doc, params);
    }, actionTypes.put, _extends({}, opts, { doc: doc }));
  };

  var remove = function remove(doc, params, opts) {
    return (0, _actions.createPromiseAction)(function () {
      return db.remove(doc, params);
    }, actionTypes.remove, _extends({}, opts, { doc: doc }));
  };

  var actions = {
    allDocs: allDocs,
    query: query,
    get: get,
    put: put,
    remove: remove
  };

  var reducer = function reducer(state, action) {
    if (state === undefined) state = INITIAL_STATE;

    switch (action.type) {
      case actionTypes.query.success:
      case actionTypes.allDocs.success:
        var type = action.type,
            folder = action.folder,
            payload = action.payload,
            folderVars = _objectWithoutProperties(action, ['type', 'folder', 'payload']);

        return utils.setQueryPayloadInState(state, folder, payload, folderVars);
      case actionTypes.put.success:
        return utils.setDocument(state, _extends({}, action.doc, {
          _rev: action.payload.rev
        }));
      case actionTypes.get.success:
        return utils.setDocument(state, action.payload);
      case actionTypes.remove.success:
        return utils.removeDocument(state, action.payload.id);
      default:
        return state;
    }
  };

  var paths = {
    list: urlPrefix + '/',
    detail: utils.createDetailLink(urlPrefix, ':id'),
    edit: urlPrefix + '/:id/edit/',
    create: urlPrefix + '/new/'
  };

  return {
    actions: actions,
    actionTypes: actionTypes,
    reducer: reducer,
    mountPoint: mountPoint,
    paths: paths,
    urlPrefix: urlPrefix,
    db: db,
    startkey: startkey,
    endkey: endkey
  };
}